/*********************************************************************

FileName:        MACAPI.h

Header file for MAC-layer API 

This firmware may only be used with FlexiPanel Pixie products.

********************************************************************/

typedef enum _ZIGBEE_PRIMITIVE
{
    NO_PRIMITIVE = 0,

	// PHY-layer primitives
	// with the exception of PD_DATA_indication in promiscuous mode, these are for internal use only 

    PD_DATA_request = 0x01,
    PD_DATA_confirm = 0x02,
    PD_DATA_indication = 0x03,
    PLME_CCA_request = 0x04,
    PLME_CCA_confirm = 0x05,
    PLME_ED_request = 0x06,
    PLME_ED_confirm = 0x07,
    PLME_GET_request = 0x08,
    PLME_GET_confirm = 0x09,
    PLME_SET_TRX_STATE_request = 0x0A,
    PLME_SET_TRX_STATE_confirm = 0x0B,
    PLME_SET_request = 0x0C,
    PLME_SET_confirm = 0x0D,

	// MAC-layer primitives 

    MCPS_DATA_request = 0x10,
    MCPS_DATA_confirm = 0x11,
    MCPS_DATA_indication = 0x12,
    MCPS_PURGE_request = 0x13,
    MCPS_PURGE_confirm = 0x14,
    MLME_ASSOCIATE_request = 0x15,
    MLME_ASSOCIATE_indication = 0x16,
    MLME_ASSOCIATE_response = 0x17,
    MLME_ASSOCIATE_confirm = 0x18,
    MLME_DISASSOCIATE_request = 0x19,
    MLME_DISASSOCIATE_indication = 0x1A,
    MLME_DISASSOCIATE_confirm = 0x1B,
    MLME_BEACON_NOTIFY_indication = 0x1C,
    MLME_GET_request = 0x1D,
    MLME_GET_confirm = 0x1E,
    MLME_GTS_request = 0x1F,
    MLME_GTS_confirm = 0x20,
    MLME_GTS_indication = 0x21,
    MLME_ORPHAN_indication = 0x22,
    MLME_ORPHAN_response = 0x23,
    MLME_RESET_request = 0x24,
    MLME_RESET_confirm = 0x25,
    MLME_RX_ENABLE_request = 0x26,
    MLME_RX_ENABLE_confirm = 0x27,
    MLME_SCAN_request = 0x28,
    MLME_SCAN_confirm = 0x29,
    MLME_COMM_STATUS_indication = 0x2A,
    MLME_SET_request = 0x2B,
    MLME_SET_confirm = 0x2C,
    MLME_START_request = 0x2D,
    MLME_START_confirm = 0x2E,
    MLME_SYNC_request = 0x2F,
    MLME_SYNC_LOSS_indication = 0x30,
    MLME_POLL_request = 0x31,
    MLME_POLL_confirm = 0x32,

} ZIGBEE_PRIMITIVE;

//------------------------------------------------------------------------

// Attributes

#define ATTR_phyCurrentChannel				0x00	
#define ATTR_phyTransmitPower				0x02	
#define ATTR_phyCCAMode						0x03	
#define ATTR_macAckWaitDuration				0x40	
#define ATTR_macAssociationPermit			0x41	
#define ATTR_macBattLifeExt					0x43	
#define ATTR_macBattLifeExtPeriods			0x44		
#define ATTR_macBeaconPayload				0x45		
#define ATTR_macBeaconPayloadLength			0x46	
#define ATTR_macBeaconOrder					0x47	
#define ATTR_macCoordExtendedAddress		0x4A		
#define ATTR_macCoordShortAddress			0x4B		
#define ATTR_macDSN							0x4C		
#define ATTR_macMaxCSMABackoffs				0x4E	
#define ATTR_macMinBE						0x4F	
#define ATTR_macPANId						0x50	
#define ATTR_macPromiscuousMode				0x51	
#define ATTR_macShortAddress				0x53	
#define ATTR_macSuperframeOrder				0x54	
#define ATTR_macTransactionPersistenceTime	0x55


/*********************************************************************/

// Return codes 

#define SUCCESS 0x00
#define TRANSACTION_OVERFLOW 0xf1
#define TRANSACTION_EXPIRED 0xf0
#define CHANNEL_ACCESS_FAILURE 0xe1
#define DENIED 0xe2
#define DISABLE_TRX_FAILURE 0xe3
#define INVALID_GTS 0xe6
#define NO_ACK 0xe9
#define UNAVAILABLE_KEY 0xf3
#define FRAME_TOO_LONG 0xe5
#define FAILED_SECURITY_CHECK 0xe4
#define INVALID_PARAMETER 0xe8
#define MAC_INVALID_HANDLE 0xe7
#define NO_DATA 0xeb
#define NO_SHORT_ADDRESS 0x3c
#define MAC_NO_BEACON 0xea

// Associate response reasons
#define ASSOCIATION_SUCCESS			0x00
#define ASSOCIATION_PAN_AT_CAPACITY 0x01
#define ASSOCIATION_DENIED			0x02

#define ERR_STACK_NOT_IDLE  		0x01		// Stack was not idle when a _Request was issued
#define ERR_MEMORY_ERROR			0x02		// ran out of heap memory 
#define ERR_PROG( x )				(0x80 + x ) // Internal error if > 0x80 - contect FlexiPanel quoting error number

/*********************************************************************/
// variable definitions 

#define ROM  rom
typedef enum _BOOL { FALSE = 0, TRUE } BOOL;
typedef unsigned char BYTE;                 // 8-bit
typedef unsigned short int WORD;            // 16-bit
typedef unsigned short long THREEBYTE;      // 24-bit
typedef unsigned long DWORD;                // 32-bit


typedef struct _LONG_ADDR
{
    BYTE v[8];
} LONG_ADDR;

typedef union _SHORT_ADDR
{
    struct _SHORT_ADDR_bits
    {
        BYTE LSB;
        BYTE MSB;
    } byte;
    WORD Val;
    BYTE v[2];
} SHORT_ADDR;

typedef union _ADDR
{
    LONG_ADDR LongAddr;
    SHORT_ADDR ShortAddr;
    BYTE v[8];
} ADDR;
typedef SHORT_ADDR PAN_ADDR;

typedef union _WORD_VAL
{
    BYTE v[2];
    WORD Val;
    struct
    {
        BYTE LSB;
        BYTE MSB;
    } byte;
} WORD_VAL;

typedef union _THREEBYTE_VAL
{
    BYTE v[3];
    THREEBYTE Val;
    struct
    {
        BYTE LSB;
        BYTE MidB;
        BYTE MSB;
    } byte;
} THREEBYTE_VAL;

#define LSB(a)          ((a).v[0])
#define MSB(a)          ((a).v[1])

typedef union _TICK
{
    DWORD Val;
    struct _TICK_bytes
    {
        BYTE b0;
        BYTE b1;
        BYTE b2;
        BYTE b3;
    } byte;
    BYTE v[4];
} TICK;

typedef union _DWORD_VAL
{
    DWORD Val;
    struct
    {
        BYTE LOLSB;
        BYTE LOMSB;
        BYTE HILSB;
        BYTE HIMSB;
    } byte;
    struct
    {
        WORD LSW;
        WORD MSW;
    } word;
    BYTE v[4];
} DWORD_VAL;


typedef union _ASSOCIATE_CAPABILITY_INFO
{
    BYTE Val;
    struct _CapBits
    {
        unsigned int AlternatePANCoordinator :1;
        unsigned int DeviceType :1;
        unsigned int PowerSource :1;
        unsigned int RxOnWhenIdle :1;
        unsigned int :2;
        unsigned int SecurityCapability :1;
        unsigned int AllocateAddress :1;
    } CapBits;
} ASSOCIATE_CAPABILITY_INFO;

typedef union _SUPERFRAME_SPEC
{
    WORD Val;
    struct _SUPERFRAME_SPEC_bytes
    {
        BYTE LSB;
        BYTE MSB;
    } byte;
    struct _SUPERFRAME_SPEC_bits
    {
        unsigned int BeaconOrder :4;
        unsigned int SuperframeOrder :4;
        unsigned int FinalCAPSlot :4;
        unsigned int BatteryLifeExtension :1;
        unsigned int :1;
        unsigned int PANCoordinator :1;
        unsigned int AssociationPermit :1;
    } bits;
} SUPERFRAME_SPEC;

typedef struct _PAN_DESCRIPTOR
{
    BYTE                    CoordAddrMode   : 1;    // spec uses 0x02 and 0x03, we'll use 0 and 1 (short/long)
    BYTE                    GTSPermit       : 1;
    BYTE                    SecurityUse     : 1;
    BYTE                    SecurityFailure : 1;
    BYTE                    ACLEntry        : 4;

    BYTE                    LogicalChannel;
    PAN_ADDR                CoordPANId;
    ADDR                    CoordAddress;
    SUPERFRAME_SPEC         SuperframeSpec;
    TICK                    TimeStamp;
    BYTE                    LinkQuality;
    struct _PAN_DESCRIPTOR  *next;
} PAN_DESCRIPTOR;

typedef union _MLME_START_fields
{
    BYTE    Val;
    struct __MLME_START_bits
    {
        BYTE    PANCoordinator          : 1;
        BYTE    BatteryLifeExtension    : 1;
        BYTE    CoordRealignment        : 1;
        BYTE    SecurityEnable          : 1;
    } bits;
} MLME_START_fields;

// MLME_SCAN_request.ScanType values
#define MAC_SCAN_ENERGY_DETECT      0x00
#define MAC_SCAN_ACTIVE_SCAN        0x01
#define MAC_SCAN_PASSIVE_SCAN       0x02
#define MAC_SCAN_ORPHAN_SCAN        0x03

// MCPS_DATA_request.FrameType values
#define FRAME_BEACON        0b00000000
#define FRAME_DATA          0b00000001
#define FRAME_ACK           0b00000010
#define FRAME_MAC_COMMAND   0b00000011

typedef union _PARAMS
{
    /* START OF PHY */

    struct _PD_DATA_request
    {
        BYTE    psduLength;
        BYTE    filler1[1];
        BYTE    *psdu;    /* in TXBuffer already */
    } PD_DATA_request;

    struct _PD_DATA_confirm
    {
        BYTE    status;
    } PD_DATA_confirm;

    struct _PD_DATA_indication
    {
        BYTE    psduLength;
        BYTE    filler1[1];
        BYTE    *psdu;    /* in RXBuffer already */
        BYTE    filler2[23];
        BYTE    ppduLinkQuality;
    } PD_DATA_indication;

//    struct _PLME_CCA_request
//    {
//        No Inputs
//    } PLME_CCA_request;

    struct _PLME_CCA_confirm
    {
        BYTE    status;
    } PLME_CCA_confirm;

//    struct _PLME_ED_request
//    {
//        No Inputs
//    } PLME_ED_request;

    struct _PLME_ED_confirm
    {
        BYTE    status;
        BYTE    filler1[26];
        BYTE    EnergyLevel;
    } PLME_ED_confirm;

    struct _PLME_GET_request
    {
        BYTE    filler1[1];
        BYTE    PIBAttribute;
    } PLME_GET_request;

    struct _PLME_GET_confirm
    {
        BYTE    status;
        BYTE    PIBAttribute;
        void    *PIBAttributeValue;
    } PLME_GET_confirm;

    struct _PLME_SET_TRX_STATE_request
    {
        BYTE    state;
    } PLME_SET_TRX_STATE_request;

    struct _PLME_SET_TRX_STATE_confirm
    {
        BYTE    status;
    } PLME_SET_TRX_STATE_confirm;

    struct _PLME_SET_request
    {
        BYTE    filler1[1];
        BYTE    PIBAttribute;
        void    *PIBAttributeValue;
    } PLME_SET_request;

    /* END OF PHY */

    /* START OF MAC */
    
    struct _MLME_GET_request
    {
        BYTE    MIBAttribute;
    } MLME_GET_request;

    struct _MLME_GET_confirm
    {
        BYTE    status;
        BYTE    MIBAttribute;
        BYTE    MIBAttributeLen;
        void    *MIBAttributeValue;
    } MLME_GET_confirm;

    struct _MLME_SET_request
    {
        BYTE    MIBAttribute;
        void    *MIBAttributeValue;
    } MLME_SET_request;
    
    struct _MLME_SET_confirm
    {
        BYTE    status;
        BYTE    MIBAttribute;
    } MLME_SET_confirm;

    struct _MLME_POLL_request
    {
        BYTE        filler1[1];
        BOOL        SecurityEnabled;
        BYTE        filler2[5];
        BYTE        CoordAddrMode;        
        BYTE        filler3[8];
        ADDR        CoordAddress;         
        PAN_ADDR    CoordPANId;          
    } MLME_POLL_request;

    struct _MLME_POLL_confirm
    {
        BYTE    status;                 
    } MLME_POLL_confirm;

    struct _MCPS_DATA_request
    {
        BYTE        msduLength;
        BYTE        filler1[1];
        BYTE        frameType; /* this is an addition to the IEEE spec to reuse the most code, either FRAME_DATA, FRAME_BEACON, FRAME_ACK, or FRAME_MAC_COMMAND */
        union _TxOptions
        {
            BYTE Val;
            struct _bits_for_MCPS_DATA_request
            {
                unsigned int acknowledged_transmission :1;
                unsigned int GTS_transmission :1;
                unsigned int indirect_transmission :1;
                unsigned int security_enabled_transmission :1;
                unsigned int :4;
            } bits;
        } TxOptions;
        PAN_ADDR    SrcPANId;
        BYTE        SrcAddrMode;
        BYTE        DstAddrMode;
        ADDR        SrcAddr;
        ADDR        DstAddr;           
        PAN_ADDR    DstPANId;
        BYTE        msduHandle;
        /* msdu is in TXBuffer */
    } MCPS_DATA_request;

    struct _MCPS_DATA_confirm
    {
        BYTE        status;                
        BYTE        filler1[25];
        BYTE        msduHandle;
    } MCPS_DATA_confirm;

    struct _MCPS_DATA_indication
    {
        BYTE        msduLength;         
        BOOL        SecurityUse;      
        BYTE*       msdu;              
        PAN_ADDR    SrcPANId;
        BYTE        SrcAddrMode;       
        BYTE        DstAddrMode;
        ADDR        SrcAddr;
        ADDR        DstAddr;
        PAN_ADDR    DstPANId;
        BYTE        ACLEntry;      
        BYTE        mpduLinkQuality;    
    } MCPS_DATA_indication;

    struct _MCPS_PURGE_request
    {
        BYTE        filler1[26];
        BYTE        msduHandle;
    }   MCPS_PURGE_request;

    struct _MCPS_PURGE_confirm
    {
        BYTE        status;
        BYTE        filler1[25];
        BYTE        msduHandle;
    }   MCPS_PURGE_confirm;

    struct _MLME_ORPHAN_indication
    {
        BYTE        filler1[1];
        BOOL        SecurityUse;      
        BYTE        filler2[6];
        LONG_ADDR   OrphanAddress;   
        BYTE        filler3[10];
        BYTE        ACLEntry;      
    } MLME_ORPHAN_indication;

    struct _MLME_ORPHAN_response
    {
        BOOL        AssociatedMember;
        BOOL        SecurityEnable;
        BYTE        filler1[6];
        LONG_ADDR   OrphanAddress;     
        SHORT_ADDR  ShortAddress;
    } MLME_ORPHAN_response;

    struct _MLME_DISASSOCIATE_request
    {
        BYTE        DisassociateReason;
        BOOL        SecurityUse;     
        BYTE        filler1[14];
        LONG_ADDR   DeviceAddress;     
    } MLME_DISASSOCIATE_request;

    struct _MLME_DISASSOCIATE_indication
    {
        BYTE        DisassociateReason;
        BOOL        SecurityUse;      
        BYTE        filler1[6];
        LONG_ADDR   DeviceAddress;     
        BYTE        filler2[10];
        BYTE        ACLEntry;       
    } MLME_DISASSOCIATE_indication;

    struct _MLME_DISASSOCIATE_confirm
    {
        BYTE        status;
    } MLME_DISASSOCIATE_confirm;

    struct _MLME_SYNC_LOSS_indication
    {
        BYTE        LossReason;
    } MLME_SYNC_LOSS_indication;

    struct _MLME_BEACON_NOTIFY_indication
    {
        BYTE            sduLength;    
        BOOL            SecurityUse;     
        BYTE            *sdu;       
        PAN_ADDR        CoordPANId;    
        BYTE            CoordAddrMode;   
        BYTE            LogicalChannel;
        ADDR            CoordAddress;   
        TICK            TimeStamp;
        BYTE            BSN;
        BYTE            filler1[1];
        BOOL            SecurityFailure;
        BOOL            GTSPermit;
        SUPERFRAME_SPEC SuperframeSpec;
        BYTE            ACLEntry;      
        BYTE            LinkQuality;    

// we don't use AddrList because we have a non-beacon network only.
//        BYTE* AddrList;
// We don't use PendAddrSpec because we use a non-beacon network
//        union PENDING_ADDR
//        {
//            BYTE Val;
//            struct _PENDING_ADDR_bits
//            {
//                unsigned int numShortAddresses :3;
//                unsigned int :1;
//                unsigned int numExtAddresses :3;
//                unsigned int :1;
//            } bits;
//        } PendAddrSpec;

    } MLME_BEACON_NOTIFY_indication;

    struct _MLME_ASSOCIATE_indication
    {
        ASSOCIATE_CAPABILITY_INFO   CapabilityInformation;
        BOOL                        SecurityUse;
        BYTE                        filler1[6];
        LONG_ADDR                   DeviceAddress; 
        BYTE                        filler2[10];
        BYTE                        ACLEntry;
    } MLME_ASSOCIATE_indication;

    struct _MLME_ASSOCIATE_confirm
    {
        BYTE        status;                   
        BYTE        filler1[5];
        SHORT_ADDR  AssocShortAddress;       
    } MLME_ASSOCIATE_confirm;

    struct _MLME_ASSOCIATE_request
    {
        ASSOCIATE_CAPABILITY_INFO   CapabilityInformation; 
        BOOL                        SecurityEnable;           
        BYTE                        filler1[4];
        BYTE                        LogicalChannel;
        BYTE                        CoordAddrMode;
        BYTE                        filler2[8];
        ADDR                        CoordAddress;       
        PAN_ADDR                    CoordPANId;          
    } MLME_ASSOCIATE_request;

    struct _MLME_ASSOCIATE_response
    {
        BYTE                        status;
        BOOL                        SecurityEnable;        
        BYTE                        filler1[4];
        SHORT_ADDR                  AssocShortAddress;
        BYTE                        filler2[8];
        LONG_ADDR                   DeviceAddress;         
    } MLME_ASSOCIATE_response;

    struct _MLME_COMM_STATUS_indication
    {
        BYTE        status;
        BYTE        filler1[3];
        PAN_ADDR    PANId;              //not populated because its always macPIB.macPANId for Zigbee 1.0
        BYTE        SrcAddrMode;        //not populated because its always 0x03 for zigbee
        BYTE        DstAddrMode;        //not populated because its always 0x03 for Zigbee
        ADDR        SrcAddr;        //not populated because its always MAC_LONG_ADDR for Zigbee
        ADDR        DstAddr;         
    } MLME_COMM_STATUS_indication;

    struct _MLME_RESET_request
    {
        BOOL        SetDefaultPIB;
    } MLME_RESET_request;

    struct _MLME_RESET_confirm
    {
        BYTE        status;
    } MLME_RESET_confirm;

    struct _MLME_SCAN_request
    {
        BYTE                        filler1[6];
        BYTE                        ScanType;
        BYTE                        ScanDuration;              
        DWORD_VAL                   ScanChannels;            
    } MLME_SCAN_request;

    struct _MLME_SCAN_confirm
    {
        BYTE                        status;                  
        BYTE                        ResultListSize;
        BYTE                        *EnergyDetectList;
        PAN_DESCRIPTOR              *PANDescriptorList;
        BYTE                        ScanType;
        BYTE                        filler1[1];
        DWORD_VAL                   UnscannedChannels;
    } MLME_SCAN_confirm;


    struct _MLME_START_request
    {
        MLME_START_fields           fields;
        BYTE                        filler1[3];
        PAN_ADDR                    PANId;
        BYTE                        LogicalChannel;
        BYTE                        filler2[5];
        BYTE                        BeaconOrder;
        BYTE                        SuperframeOrder;
    } MLME_START_request;

    struct _MLME_START_confirm
    {
        BYTE                        status;            
    } MLME_START_confirm;


} PARAMS;


// Parameters variable for stack 
extern PARAMS           params;

/*********************************************************************/

// Macros
typedef struct _ZIGBEE_STATUS
{
    ZIGBEE_PRIMITIVE    nextZigBeeState;
    union _ZIGBEE_STATUS_FLAGS
    {
        BYTE    Val;
        struct _ZIGBEE_STATUS_BITS
        {
            BYTE        bTxFIFOInUse            : 1;
            BYTE        bRxBufferOverflow       : 1;
            //BYTE        bRxHardwareOverflow     : 1; not used
            BYTE        bHasBackgroundTasks     : 1;
            #ifdef I_AM_COORDINATOR
                BYTE    bNetworkFormed          : 1;
                BYTE    bTryingToFormNetwork    : 1;
            #else
                BYTE    bNetworkJoined          : 1;
                BYTE    bTryingToJoinNetwork    : 1;
                BYTE    bTryOrphanJoin          : 1;
            #endif
            #ifdef I_AM_RFD
                BYTE    bRequestingData         : 1;
                BYTE    bDataRequestComplete    : 1;
            #endif
        } bits;
    } flags;
} ZIGBEE_STATUS;
extern volatile ZIGBEE_STATUS  	ZigBeeStatus;
#ifdef I_AM_SLEEPY
	#define IsMACReady()		(!ZigBeeStatus.flags.bits.bTxFIFOInUse && ZigBeeStatus.flags.bits.bDataRequestComplete)
#else
	#define IsMACReady()		( !ZigBeeStatus.flags.bits.bTxFIFOInUse )
#endif
#define RANDOM_LSB                      (TMR0L)
#define RANDOM_MSB                      (TMR0H)
#define MACBlockTx()     		ZigBeeStatus.flags.bits.bTxFIFOInUse = 1

// StackCanSleep() should only be used in the case of 
#define StackCanSleep()			(!ZigBeeStatus.flags.bits.bHasBackgroundTasks)

// Buffer locations
#define pTxData  		(&TxBuffer[TX_DATA_START])
extern BYTE             TxBuffer[];
#define TX_DATA_START   0

#define DISABLE_WDT()       (WDTCONbits.SWDTEN = 0)
#define ENABLE_WDT()        (WDTCONbits.SWDTEN = 1)
#define RCIP                IPR1bits.RCIP  
#define TXIP                IPR1bits.TXIP  
#define TXIE                PIE1bits.TXIE
#define RCIE                PIE1bits.RCIE
#define RBIF                INTCONbits.RBIF
#define RBIE                INTCONbits.RBIE
#define RCIF                PIR1bits.RCIF
#define TXIF                PIR1bits.TXIF
#define CREN                RCSTAbits.CREN
#define OERR                RCSTAbits.OERR
#define FERR                RCSTAbits.FERR  
#define TRMT                TXSTAbits.TRMT

/*********************************************************************/

// API function calls

BOOL MACAPITasks( ZIGBEE_PRIMITIVE *currentPrimitive );
void MACAPIInit( void );

/*********************************************************************/

// API function callbacks

void MACAPIHook( void );
void UserInterruptHandler(void);
void PriorityUserInterruptHandler(void);
 
/*********************************************************************/

// attributes 

extern ROM LONG_ADDR    macLongAddr;
	
extern ROM unsigned char IsSQTPed;     // non-zero if MAC address has been set
#define IsMACAddressValid()				(IsSQTPed)

#define GetMACAddress( x )              NVMRead( x, (ROM void*)&macLongAddr, sizeof(LONG_ADDR) )
#define GetMACAddressByte( y, x )       NVMRead( x, (ROM void*)((int)&macLongAddr + (int)y), 1 )
#define SetMACAddress( x )              { BYTE IsSet = 0x01; NVMWrite((NVM_ADDR*)&macLongAddr, (BYTE*)x, sizeof(LONG_ADDR)); NVMWrite((NVM_ADDR*)&IsSQTPed, (BYTE*)&IsSet, sizeof(BYTE)); }

/*********************************************************************/

// Utility functions

// nonvolatile memory read and write
// memory is checked before writing, so repeated writing the same value does not exhaust flash
typedef ROM BYTE NVM_ADDR;
void NVMWrite(NVM_ADDR *dest, BYTE *src, BYTE count);
#define NVMRead(dest, src, count)   memcpypgm2ram(dest, src, count)
 
// 4-byte timer in 62500ths of a second (i.e. rolls over after 19 hours)
TICK TickGet(void);
#define TickGetDiff(a,b) (a.Val - b.Val)
#ifndef ONE_SECOND
#define ONE_SECOND ((DWORD)62500)
#endif

// dynamic memory.  maximum allocation 126 bytes
unsigned char * SRAMalloc(unsigned char nBytes);  //  returns 0 if can't allocate memory
void SRAMfree(unsigned char * pSRAM);

void MACDiscardRx(void);			// used for freeing packets from PD_DATA_indication, MCPS_DATA_indication, MLME_BEACON_indicatioon
void MACEnable(void);				// turn off CC2420 prior to sleeping
void MACDisable(void);				// turn on CC2420 prior after sleeping

// set bNoTimeout to non-zero if you want to slow timeouts in a similar manner to the PixieMAC +DHTR command
extern BYTE bNoTimeout;

/*********************************************************************/

// Attributes
// These may be set or read directly on one condition - that if you 
// change the short address or PANid, you call MLME_SET_macPANId_hw or
// MLME_SET_macShortAddress_hw afterwards as appropriate

#define MAC_PIB_macBeaconPayloadLength 3
typedef struct _MAC_PIB
{
//    BYTE macAckWaitDuration;  //made a constant
    unsigned int macAssociationPermit :1;
    unsigned int macAutoRequest :1;
    unsigned int macBattLifeExtPeriods :1;
    unsigned int macPromiscuousMode :1;
    unsigned int macRxOnWhenIdle :1;
    BYTE macBeaconPayload[MAC_PIB_macBeaconPayloadLength];   /* always 3 for non-beacon, 5 for beacon */
//    BYTE macBeaconPayloadLength;  /* made a constant */
//    BYTE macBeaconOrder;   //made a constant
    TICK macBeaconTxTime;
    BYTE macBSN;
    LONG_ADDR macCoordExtendedAddress;
    SHORT_ADDR macCoordShortAddress;
    BYTE macDSN;
    BYTE macMaxCSMABackoffs;
    BYTE macMinBE;
    PAN_ADDR macPANId;
    SHORT_ADDR macShortAddress;
//    BYTE macSuperframeOrder;  //made a constant
//    WORD macTransactionPersistenceTime; //made nonvolatile
} MAC_PIB;
extern MAC_PIB macPIB;
extern ROM DWORD MIB_PersistTime;

#define SetPersistTime( lVal ) NVMWrite((NVM_ADDR*)&MIB_PersistTime, (BYTE*)&lVal, sizeof(MIB_PersistTime));

void MLME_SET_macPANId_hw( void );
void MLME_SET_macShortAddress_hw( void );


/*********************************************************************/
